;**************************** Librera "DS18B20.INC" ***********************************
;
; Estas subrutinas permiten realizar las tareas de manejo del sensor de temperatura DS18B20.
; Este sensor transmite la informacin va serie a travs de un 1-Wire Bus.
;
;***************************************************************************************

	CBLOCK
	DS18B20_Temperatura						; Primer byte ledo del termmetro DS18B20.
	DS18B20_TemperaturaH					; Segundo byte ledo del termmetro DS18B20.
	DS18B20_TemperaturaSigno				; Indica el signo de la temperatura.
	DS18B20_TemperaturaDecimal				; Parte decimal de la temperatura medida.
	ENDC

DS18B20_Skip_ROM_Command			EQU	0xCC	; Comandos del DS18B20.
DS18B20_Read_Scratchpad_Command		EQU	0xBE
DS18B20_Convert_T_Command			EQU	0x44

; Subrutina "DS18B20_RedondeaDecimal" ----------------------------------------------------

DS18B20_Redondeo
	addwf	PCL,F	
	retlw	d'0'	; (DS18B20_Decimal)=b'0000'. Mide 0,0000C. Redondeado a 0,0C.
	retlw	d'1'	; (DS18B20_Decimal)=b'0001'. Mide 0,0625C. Redondeado a 0,1C.
	retlw	d'1'	; (DS18B20_Decimal)=b'0010'. Mide 0,1250C. Redondeado a 0,1C.
	retlw	d'2'	; (DS18B20_Decimal)=b'0011'. Mide 0,1875C. Redondeado a 0,2C.
	retlw	d'3'	; (DS18B20_Decimal)=b'0100'. Mide 0,2500C. Redondeado a 0,3C.
	retlw	d'3'	; (DS18B20_Decimal)=b'0101'. Mide 0,3125C. Redondeado a 0,3C.	
	retlw	d'4'	; (DS18B20_Decimal)=b'0110'. Mide 0,3750C. Redondeado a 0,4C.
	retlw	d'4'	; (DS18B20_Decimal)=b'0111'. Mide 0,4375C. Redondeado a 0,4C.
	retlw	d'5'	; (DS18B20_Decimal)=b'1000'. Mide 0,5000C. Redondeado a 0,5C.
	retlw	d'6'	; (DS18B20_Decimal)=b'1001'. Mide 0,5625C. Redondeado a 0,6C.
	retlw	d'6'	; (DS18B20_Decimal)=b'1010'. Mide 0,6250C. Redondeado a 0,6C.
	retlw	d'7'	; (DS18B20_Decimal)=b'1011'. Mide 0,6875C. Redondeado a 0,7C.	
	retlw	d'8'	; (DS18B20_Decimal)=b'1100'. Mide 0,7500C. Redondeado a 0,8C.
	retlw	d'8'	; (DS18B20_Decimal)=b'1101'. Mide 0,8125C. Redondeado a 0,8C.
	retlw	d'9'	; (DS18B20_Decimal)=b'1110'. Mide 0,8750C. Redondeado a 0,9C.
	retlw	d'9'	; (DS18B20_Decimal)=b'1111'. Mide 0,9375C. Redondeado a 0,9C.

DS18B20_Inicializa
	call	Bus1Linea_Reset				; Primero los pulsos de Reset y Presence.
	movlw	DS18B20_Skip_ROM_Command 	; Salta los comandos de ROM ya que hay
	call	Bus1Linea_EscribeByte		; conectado un nico dispositivo al bus.
	movlw	DS18B20_Convert_T_Command	; Para que comience la conversin de temperatura.
	call	Bus1Linea_EscribeByte
	return

DS18B20_LeeTemperatura
	call	Bus1Linea_Reset				; Primero los pulsos de "Reset" y "Presence".
	movlw	DS18B20_Skip_ROM_Command 	; Salta los comandos de ROM ya que hay
	call	Bus1Linea_EscribeByte		; conectado un nico dispositivo al bus.
	movlw	DS18B20_Read_Scratchpad_Command ; Va a leer la primera posicin de memoria,
	call	Bus1Linea_EscribeByte		; donde se localiza el valor de la temperatura.
	call	Bus1Linea_LeeByte			; Lee la temperatura en 
	movwf	DS18B20_Temperatura			; complemento a 2.
	call	Bus1Linea_LeeByte			; Ahora lee el segundo byte
	movwf	DS18B20_TemperaturaH		; de la temperatura.
	call	Bus1Linea_Reset				; Y para terminar, resetea el dispositivo.

; Ahora deduce si la temperatura es positiva o negativa y halla su valor absoluto.

	btfss	DS18B20_TemperaturaH,7		; Si el bit es "1": temperatura negativa.
	goto	DS18B20_FinLeeTemperatura	; La temperatura es positiva y salta.
DS18B20_TemperaturaNegativa
	comf	DS18B20_TemperaturaH,F		; Obtiene el valor absoluto de la temperatura.
	comf	DS18B20_Temperatura,F
	movlw	.1
	addwf	DS18B20_Temperatura,F
	btfsc	STATUS,C
	incf	DS18B20_TemperaturaH,F
	movlw	0xFF
	movwf	DS18B20_TemperaturaSigno
DS18B20_FinLeeTemperatura
	movf	DS18B20_Temperatura,W		; Se queda con la parte decimal que es el nible
	andlw	b'00001111'					; bajo del primer byte.
	call	DS18B20_Redondeo		; Obtiene el valor redondeado.
	movwf	DS18B20_TemperaturaDecimal	; La parte decimal.

; Ahora va a obtener el valor de la temperatura en un solo byte.

	movlw	b'11110000'					; Se queda con el nible alto del primer byte ledo.
	andwf	DS18B20_Temperatura,F		;
	movlw	b'00001111'					; Ahora el nible bajo del segundo byte ledo.
	andwf	DS18B20_TemperaturaH,W
	iorwf	DS18B20_Temperatura,F		; Una OR con los dos nibles.
	swapf	DS18B20_Temperatura,F		; Se intercambian los nibles.
	movf	DS18B20_Temperatura,W		; En (W) la parte entera del valor absoluto de la
	return								; temperatura.
	
